@@ -18,7 +18,8 @@ from .db import *
 from .dictionary import *
 from .api import LanguageServer
 from . import __version__
-
+from .ext.reader import ReaderServer
+from .ext.importer import KindleImporter
 
 # If on macOS, display the modifier key as "Cmd", else display it as "Ctrl"
 if platform.system() == "Darwin":
@@ -68,6 +69,7 @@ class DictionaryWindow(QMainWindow):
             self.setupWidgetsV()
         else:
             self.setupWidgetsH()
+        self.setupMenu()
         self.setupButtons()
         self.startServer()
         self.initTimer()
@@ -91,7 +93,7 @@ class DictionaryWindow(QMainWindow):
     def initWidgets(self):
         self.namelabel = QLabel("Simple Sentence Mining v" + __version__)
         self.namelabel.setFont(QFont("Sans Serif", QApplication.font().pointSize() * 1.5))
-
+        self.menu = QMenuBar(self)
         self.sentence = MyTextEdit()
         self.sentence.setMinimumHeight(30)
         self.sentence.setMaximumHeight(130)
@@ -180,6 +182,31 @@ class DictionaryWindow(QMainWindow):
 
         self.bar.addPermanentWidget(self.stats_label)
 
+    def setupMenu(self):
+        self.open_reader_action = QAction("&Reader")
+        self.menu.addAction(self.open_reader_action)
+        if not self.settings.value("reader_enabled", True, type=bool):
+            self.open_reader_action.setEnabled(False)
+        importmenu = self.menu.addMenu("&Import")
+        self.help_action = QAction("Help")
+        self.menu.addAction(self.help_action)
+    
+        self.import_koreader_action = QAction("Import K&OReader")
+        self.import_koreader_action.setEnabled(False)
+        self.import_kindle_action = QAction("Import &Kindle")
+
+        self.help_action.triggered.connect(self.onHelp)
+        self.open_reader_action.triggered.connect(self.onReaderOpen)
+        self.import_kindle_action.triggered.connect(self.importkindle)
+        
+        importmenu.addActions([self.import_koreader_action, self.import_kindle_action])
+
+        self.setMenuBar(self.menu)
+
+    def onHelp(self):
+        url = f"https://freelanguagetools.org/2021/07/simple-sentence-mining-ssmtool-full-tutorial/"
+        QDesktopServices.openUrl(QUrl(url))
+
     def setupWidgetsH(self):
         self.layout = QGridLayout(self.widget)
         self.sentence.setMaximumHeight(1300)
@@ -236,6 +263,23 @@ class DictionaryWindow(QMainWindow):
         self.settings_dialog = SettingsDialog(self)
         self.settings_dialog.exec()
 
+    def importkindle(self):
+        #fdialog = QFileDialog()
+        #fdialog.setFileMode(QFileDialog.ExistingFile)
+        #fdialog.setAcceptMode(QFileDialog.AcceptOpen)
+        #fdialog.setNameFilter("Kindle clippings files (*.txt)")
+        #fdialog.exec()
+        fname = QFileDialog.getOpenFileName(
+            parent=self,
+            caption="Select a file",
+            filter='Kindle clippings files (*.txt)',
+            )[0]
+        if not fname:
+            return
+        else:
+            self.import_kindle = KindleImporter(self, fname)
+            self.import_kindle.exec()
+
     def setupShortcuts(self):
         self.shortcut_toanki = QShortcut(QKeySequence('Ctrl+S'), self)
         self.shortcut_toanki.activated.connect(self.toanki_button.animateClick)
@@ -259,6 +303,10 @@ class DictionaryWindow(QMainWindow):
         url = self.settings.value("custom_url").replace("@@@@", self.word.text())
         QDesktopServices.openUrl(QUrl(url))
 
+    def onReaderOpen(self):
+        url = f"http://{self.settings.value('reader_host', type=str)}:{self.settings.value('reader_port', type=str)}"
+        QDesktopServices.openUrl(QUrl(url))
+
     def lookupClicked(self, use_lemmatize=True):
         if self.forvo_scraping:
             return
@@ -324,6 +372,11 @@ class DictionaryWindow(QMainWindow):
             self.setSentence(preprocess_clipboard(text, lang))
 
     def lookupSet(self, word, use_lemmatize=True):
+        sentence_text = self.sentence.toPlainText()
+        if self.settings.value("bold_word", type=bool) == True:
+            sentence_text = sentence_text.replace("_","").replace(word, f"__{word}__")
+        self.sentence.setText(sentence_text)
+        QCoreApplication.processEvents()
         result = self.lookup(word, use_lemmatize)
         self.setState(result)
         QCoreApplication.processEvents()
@@ -333,12 +386,12 @@ class DictionaryWindow(QMainWindow):
             self.audio_path = play_forvo(word, code[self.settings.value("target_language")])
             self.forvo_scraping = False
 
-    def lookup(self, word, use_lemmatize=True):
+    def lookup(self, word, use_lemmatize=True, record=True):
         """
         Look up a word and return a dict with the lemmatized form (if enabled)
         and definition
         """
-        TL = self.settings.value("target_language", "English") 
+        TL = self.settings.value("target_language", "English")
         self.prev_states.append(self.getState())
         lemmatize = use_lemmatize and self.settings.value("lemmatization", True, type=bool)
         lemfreq = self.settings.value("lemfreq", True, type=bool)
@@ -347,21 +400,24 @@ class DictionaryWindow(QMainWindow):
         gtrans_lang = self.settings.value("gtrans_lang", "English")
         dictname = self.settings.value("dict_source", "Wiktionary (English)")
         freqname = self.settings.value("freq_source", "Disabled")
-        word = re.sub('[«»…()\[\]]*', "", word)
+        word = re.sub('[«»…,()\[\]]*', "", word)
         if freqname != "Disabled":
             try:
                 freq = getFreq(word, language, lemfreq, freqname)
             except TypeError:
                 freq = -1
             self.freq_display.display(freq)
-        self.status(f"L: '{word}' in '{language}', lemma: {short_sign}, from {dictionaries.get(dictname, dictname)}")
+        if record:
+            self.status(f"L: '{word}' in '{language}', lemma: {short_sign}, from {dictionaries.get(dictname, dictname)}")
         try:
             item = lookupin(word, language, lemmatize, dictname, gtrans_lang)
-            self.rec.recordLookup(word, item['definition'], TL, lemmatize, dictionaries.get(dictname, dictname), True)
+            if record:
+                self.rec.recordLookup(word, item['definition'], TL, lemmatize, dictionaries.get(dictname, dictname), True)
         except Exception as e:
-            self.status(str(e))
-            self.rec.recordLookup(word, None, TL, lemmatize, dictionaries.get(dictname, dictname), False)
-            self.updateAnkiButtonState(True)
+            if record:
+                self.status(str(e))
+                self.rec.recordLookup(word, None, TL, lemmatize, dictionaries.get(dictname, dictname), False)
+                self.updateAnkiButtonState(True)
             item = {
                 "word": word,
                 "definition": failed_lookup(word, self.settings)
@@ -372,10 +428,12 @@ class DictionaryWindow(QMainWindow):
             return item
         try:
             item2 = lookupin(word, language, lemmatize, dict2name, gtrans_lang)
-            self.rec.recordLookup(word, item['definition'], TL, lemmatize, dictionaries.get(dict2name, dict2name), True)
+            if record:
+                self.rec.recordLookup(word, item['definition'], TL, lemmatize, dictionaries.get(dict2name, dict2name), True)
         except Exception as e:
             self.status("Dict-2 failed" + str(e))
-            self.rec.recordLookup(word, None, TL, lemmatize, dictionaries.get(dict2name, dict2name), False)
+            if record:
+                self.rec.recordLookup(word, None, TL, lemmatize, dictionaries.get(dict2name, dict2name), False)
             self.definition2.clear()
             return item
         return {"word": item['word'], 'definition': item['definition'], 'definition2': item2['definition']}
@@ -383,6 +441,8 @@ class DictionaryWindow(QMainWindow):
 
     def createNote(self):
         sentence = self.sentence.toPlainText().replace("\n", "<br>")
+        if self.settings.value("bold_word", type=bool) == True:
+            sentence = re.sub(r"__([ \w]+)__", r"<strong>\1</strong>", sentence)
         if self.settings.value("remove_spaces", type=bool) == True:
             sentence = re.sub("\s", "", sentence)
         tags = (self.settings.value("tags", "ssmtool").strip() + " " + self.tags.text().strip()).split(" ")
@@ -471,17 +531,33 @@ class DictionaryWindow(QMainWindow):
         msg.exec()
 
     def startServer(self):
-        try:
-            self.thread = QThread()
-            port = self.settings.value("port", 39284, type=int)
-            host = self.settings.value("host", "127.0.0.1")
-            self.worker = LanguageServer(self, host, port)
-            self.worker.moveToThread(self.thread)
-            self.thread.started.connect(self.worker.start_api)
-            self.worker.note_signal.connect(self.onNoteSignal)
-            self.thread.start()
-        except Exception:
-            self.status("Failed to start server")
+        if self.settings.value("api_enabled", True, type=bool):
+            try:
+                self.thread = QThread()
+                port = self.settings.value("port", 39284, type=int)
+                host = self.settings.value("host", "127.0.0.1")
+                self.worker = LanguageServer(self, host, port)
+                self.worker.moveToThread(self.thread)
+                self.thread.started.connect(self.worker.start_api)
+                self.worker.note_signal.connect(self.onNoteSignal)
+                self.thread.start()
+            except Exception as e:
+                print(e)
+                self.status("Failed to start API server")
+        if self.settings.value("reader_enabled", True, type=bool):
+            try:
+                self.thread2 = QThread()
+                port = self.settings.value("reader_port", 39285, type=int)
+                host = self.settings.value("reader_host", "127.0.0.1")
+                self.worker2 = ReaderServer(self, host, port)
+                self.worker2.moveToThread(self.thread2)
+                self.thread2.started.connect(self.worker2.start_api)
+                self.thread2.start()
+            except Exception as e:
+                print(e)
+                self.status("Failed to start reader server")
+            
+
     
     def onNoteSignal(self, sentence: str, word: str, definition: str, tags: list):
         self.setSentence(sentence)
